@@ -1,28 +1,28 @@
 import inspect
 from collections import OrderedDict
-from typing import Any, Callable, Dict, List
+from typing import Any, Callable, Dict, List, Set
 
 import click
 
-from apistar import commands, pipelines, routing, schema
-
+from apistar import commands as cmd
+from apistar import pipelines, routing, schema
 
 DEFAULT_LOOKUP_CACHE_SIZE = 10000
 
 
 class App(object):
     built_in_commands = (
-        commands.new,
-        commands.run,
-        commands.test,
+        cmd.new,
+        cmd.run,
+        cmd.schema,
+        cmd.test,
     )
 
     def __init__(self,
-                 routes: List[routing.Route] = None,
+                 routes: routing.RoutesConfig = None,
                  commands: List[Callable] = None,
                  settings: Dict[str, Any] = None) -> None:
         from apistar.settings import Settings
-        from apistar.templating import Templates
 
         routes = [] if (routes is None) else routes
         commands = [] if (commands is None) else commands
@@ -35,9 +35,9 @@ class App(object):
         self.preloaded = {
             'app': self
         }
-        if 'TEMPLATES' in self.settings:
-            initial_types.append(Templates)
-            self.preloaded['templates'] = Templates.build(self.settings)
+        preload_state(self.preloaded, self.routes)
+        if 'sql_alchemy' in self.preloaded:
+            self.commands += [cmd.create_tables]
 
         self.router = routing.Router(self.routes, initial_types)
         self.wsgi = get_wsgi_server(app=self)
@@ -54,7 +54,7 @@ def get_wsgi_server(app):
     preloaded = app.preloaded
 
     # Pre-fill the lookup cache for URLs without path arguments.
-    for path, method, view in app.router.routes:
+    for path, method, view in routing.walk(app.routes):
         if '{' not in path:
             key = method.upper() + ' ' + path
             lookup_cache[key] = lookup(path, method)
@@ -69,7 +69,7 @@ def get_wsgi_server(app):
             'path': path,
             'exception': None,
             'view': None,
-            'url_path_args': {}
+            'url_path_args': {},
         }
         state.update(preloaded)
 
@@ -155,3 +155,26 @@ def get_click_client(app):
         client.add_command(command)
 
     return client
+
+
+def preload_state(state: Dict[str, Any], routes: routing.RoutesConfig) -> None:
+    components = get_preloaded_components(routes)
+    for component in components:
+        builder = getattr(component, 'build')
+        pipeline = pipelines.build_pipeline(
+            function=builder,
+            initial_types=[App]
+        )
+        pipelines.run_pipeline(pipeline, state)
+
+
+def get_preloaded_components(routes: routing.RoutesConfig) -> Set[type]:
+    preloaded_components = set()
+
+    for path, method, view in routing.walk(routes):
+        view_signature = inspect.signature(view)
+        for param in view_signature.parameters.values():
+            if getattr(param.annotation, 'preload', False):
+                preloaded_components.add(param.annotation)
+
+    return preloaded_components
