@@ -5,9 +5,13 @@ from werkzeug.datastructures import Headers as WerkzeugHeaders
 from werkzeug.datastructures import (
     EnvironHeaders, ImmutableDict, ImmutableHeadersMixin, ImmutableMultiDict
 )
+from werkzeug.formparser import parse_form_data
+from werkzeug.http import parse_options_header
 from werkzeug.urls import url_decode
+from werkzeug.wsgi import get_input_stream
 
-import ujson as json
+from apistar import exceptions
+from apistar.compat import json
 from apistar.pipelines import ArgName
 from apistar.schema import validate
 
@@ -97,8 +101,7 @@ class URL(str):
 class Body(bytes):
     @classmethod
     def build(cls, environ: WSGIEnviron):
-        content_length = int(environ.get('CONTENT_LENGTH', 0))
-        return environ['wsgi.input'].read(content_length)
+        return get_input_stream(environ).read()
 
 
 class Headers(ImmutableHeadersMixin, WerkzeugHeaders):
@@ -134,7 +137,7 @@ class QueryParam(str):
         if value is None or cls.schema is None:
             return value
         if not isinstance(value, cls.schema):
-            value = validate(cls.schema, value)
+            value = validate(cls.schema, value, key=arg_name)
         return value
 
 
@@ -148,6 +151,48 @@ HeadersType = Union[
 ResponseData = TypeVar('ResponseData')
 
 
+class RequestData(object):
+    schema = None  # type: type
+
+    @classmethod
+    def build(cls, environ: WSGIEnviron):
+        if not bool(environ.get('CONTENT_TYPE')):
+            mimetype = None
+        else:
+            mimetype, _ = parse_options_header(environ['CONTENT_TYPE'])
+
+        if mimetype is None:
+            value = None
+        elif mimetype == 'application/json':
+            body = get_input_stream(environ).read()
+            value = json.loads(body.decode('utf-8'))
+        elif mimetype in ('multipart/form-data', 'application/x-www-form-urlencoded'):
+            stream, form, files = parse_form_data(environ)
+            value = ImmutableMultiDict(list(form.items()) + list(files.items()))
+        else:
+            raise exceptions.UnsupportedMediaType()
+
+        if cls.schema is None:
+            return value
+        if not isinstance(value, cls.schema):
+            value = validate(cls.schema, value)
+        return value
+
+
+class RequestField(object):
+    schema = None  # type: type
+
+    @classmethod
+    def build(cls, data: RequestData, arg_name: ArgName):
+        value = data[arg_name]  # type: ignore
+
+        if value is None or cls.schema is None:
+            return value
+        if not isinstance(value, cls.schema):
+            value = validate(cls.schema, value, key=arg_name)
+        return value
+
+
 class Request(object):
     __slots__ = ('method', 'url', 'headers')
 
@@ -157,14 +202,20 @@ class Request(object):
         self.headers = Headers(headers)
 
     @classmethod
-    def build(cls, method: Method, url: URL, headers: Headers):
+    def build(cls,
+              method: Method,
+              url: URL,
+              headers: Headers):
         return cls(method=method, url=url, headers=headers)
 
 
 class Response(object):
     __slots__ = ('data', 'content', 'status', 'headers')
 
-    def __init__(self, data: Any, status: int=200, headers: HeadersType=None) -> None:
+    def __init__(self,
+                 data: Any,
+                 status: int=None,
+                 headers: HeadersType=None) -> None:
         if headers is None:
             headers_dict = {}  # type: Union[Dict[str, str], Headers]
             headers_list = []  # type: List[Tuple[str, str]]
@@ -180,19 +231,28 @@ class Response(object):
 
         if isinstance(data, str):
             content = data.encode('utf-8')
-            content_type = 'text/html; charset=utf-8'
+            if content:
+                content_type = 'text/html; charset=utf-8'
+            else:
+                content_type = None
         elif isinstance(data, bytes):
             content = data
-            content_type = 'text/html; charset=utf-8'
+            if content:
+                content_type = 'text/html; charset=utf-8'
+            else:
+                content_type = None
         else:
             content = json.dumps(data).encode('utf-8')
             content_type = 'application/json'
 
         if 'Content-Length' not in headers_dict:
             headers_list += [('Content-Length', str(len(content)))]
-        if 'Content-Type' not in headers_dict:
+        if content_type and 'Content-Type' not in headers_dict:
             headers_list += [('Content-Type', content_type)]
 
+        if status is None:
+            status = 200 if content_type else 204
+
         self.data = data
         self.content = content
         self.status = status
