@@ -1,7 +1,10 @@
 import inspect
 import traceback
 from collections import namedtuple
-from typing import Any, Callable, Dict, List, Tuple  # noqa
+from typing import (  # noqa
+    Any, Callable, Dict, Iterator, List, Sequence, Tuple, Union
+)
+from urllib.parse import urlparse
 
 import werkzeug
 from uritemplate import URITemplate
@@ -11,20 +14,28 @@ from werkzeug.serving import is_running_from_reloader
 from apistar import exceptions, http, pipelines, schema, wsgi
 from apistar.pipelines import ArgName, Pipeline
 
-# TODO: Path
-# TODO: Redirects
-
-
 primitive_types = (
     str, int, float, bool, list, dict
 )
 
 schema_types = (
-    schema.String, schema.Integer, schema.Number, schema.Boolean
+    schema.String, schema.Integer, schema.Number, schema.Boolean,
+    schema.Enum, schema.Object
 )
 
+typing_types = (
+    List,
+)
+
+
 Route = namedtuple('Route', ['path', 'method', 'view'])
+Include = namedtuple('Include', ['path', 'routes'])
 Endpoint = namedtuple('Endpoint', ['view', 'pipeline'])
+RoutesConfig = Sequence[Union[Route, Include]]
+
+
+class Path(schema.String):
+    format = 'path'
 
 
 class URLPathArgs(dict):
@@ -49,15 +60,18 @@ RouterLookup = Tuple[Callable, Pipeline, URLPathArgs]
 
 
 class Router(object):
-    def __init__(self, routes: List[Route], initial_types: List[type]=None) -> None:
+    def __init__(self,
+                 routes: RoutesConfig,
+                 initial_types: List[type]=None) -> None:
         required_type = wsgi.WSGIResponse
+
         initial_types = initial_types or []
         initial_types += [wsgi.WSGIEnviron, URLPathArgs, Exception]
 
         rules = []
         views = {}
 
-        for (path, method, view) in routes:
+        for (path, method, view) in walk(routes):
             view_signature = inspect.signature(view)
             uritemplate = URITemplate(path)
 
@@ -76,7 +90,10 @@ class Router(object):
                 if param.annotation is inspect.Signature.empty:
                     converter = 'string'
                 elif issubclass(param.annotation, (schema.String, str)):
-                    converter = 'string'
+                    if getattr(param.annotation, 'format', None) == 'path':
+                        converter = 'path'
+                    else:
+                        converter = 'string'
                 elif issubclass(param.annotation, (schema.Number, float)):
                     converter = 'float'
                 elif issubclass(param.annotation, (schema.Integer, int)):
@@ -99,7 +116,7 @@ class Router(object):
             extra_annotations = {}  # type: Dict[str, type]
             for param in view_signature.parameters.values():
 
-                if param.annotation == inspect.Signature.empty:
+                if param.annotation is inspect.Signature.empty:
                     annotated_type = str
                 else:
                     annotated_type = param.annotation
@@ -109,11 +126,24 @@ class Router(object):
                         schema = annotated_type
                     extra_annotations[param.name] = TypedURLPathArg
                 elif (annotated_type in primitive_types) or issubclass(annotated_type, schema_types):
-                    class TypedQueryParam(http.QueryParam):
-                        schema = annotated_type
-                    extra_annotations[param.name] = TypedQueryParam
+                    if method in ('POST', 'PUT', 'PATCH'):
+                        if issubclass(annotated_type, schema.Object):
+                            class TypedDataParam(http.RequestData):
+                                schema = annotated_type
+                            extra_annotations[param.name] = TypedDataParam
+                        else:
+                            class TypedFieldParam(http.RequestField):
+                                schema = annotated_type
+                            extra_annotations[param.name] = TypedFieldParam
+                    else:
+                        class TypedQueryParam(http.QueryParam):
+                            schema = annotated_type
+                        extra_annotations[param.name] = TypedQueryParam
 
-            if 'return' not in view.__annotations__:
+            return_annotation = view_signature.return_annotation
+            if return_annotation is inspect.Signature.empty:
+                extra_annotations['return'] = http.ResponseData
+            elif issubclass(return_annotation, (schema_types, primitive_types, typing_types)):  # type: ignore
                 extra_annotations['return'] = http.ResponseData
 
             # Determine the pipeline for the view.
@@ -125,23 +155,45 @@ class Router(object):
         self.adapter = Map(rules).bind('example.com')
         self.views = views
 
-    def lookup(self, path, method) -> RouterLookup:
+    def lookup(self, path: str, method: str) -> RouterLookup:
         try:
             (name, kwargs) = self.adapter.match(path, method)
         except werkzeug.exceptions.NotFound:
             raise exceptions.NotFound()
         except werkzeug.exceptions.MethodNotAllowed:
             raise exceptions.MethodNotAllowed()
+        except werkzeug.routing.RequestRedirect as exc:
+            path = urlparse(exc.new_url).path
+            raise exceptions.Found(path)
+
         (view, pipeline) = self.views[name]
         return (view, pipeline, kwargs)
 
 
-def exception_handler(environ: wsgi.WSGIEnviron, exc: Exception) -> http.Response:
+def exception_handler(environ: wsgi.WSGIEnviron,
+                      exc: Exception) -> http.Response:
+    if isinstance(exc, exceptions.Found):
+        return http.Response('', exc.status_code, {'Location': exc.location})
+
     if isinstance(exc, exceptions.APIException):
-        return http.Response({'message': exc.message}, exc.status_code)
+        if isinstance(exc.detail, str):
+            content = {'message': exc.detail}
+        else:
+            content = exc.detail
+        return http.Response(content, exc.status_code)
 
     if is_running_from_reloader() or environ.get('APISTAR_RAISE_500_EXC'):
         raise
 
     message = traceback.format_exc()
     return http.Response(message, 500, {'Content-Type': 'text/plain; charset=utf-8'})
+
+
+def walk(routes: RoutesConfig, prefix='') -> Iterator[Route]:
+    for entry in routes:
+        if isinstance(entry, Include):
+            yield from walk(entry.routes, prefix + entry.path)
+        elif prefix:
+            yield Route(prefix + entry.path, entry.method, entry.view)
+        else:
+            yield entry
