@@ -1,3 +1,5 @@
+from typing import Any, List, Union
+
 from apistar import app
 from apistar.pipelines import ArgName
 
@@ -7,7 +9,7 @@ class Settings(dict):
     def build(cls, app: app.App):
         return cls(app.settings)
 
-    def get(self, indexes, default=None):
+    def get(self, indexes: Union[str, List[str]], default: Any=None) -> Any:
         if isinstance(indexes, str):
             return super().get(indexes, default)
 
