@@ -1,4 +1,5 @@
 import os
+from typing import List  # noqa
 
 import jinja2
 
@@ -8,18 +9,27 @@ from apistar.settings import Settings
 
 
 class Templates(jinja2.Environment):
+    preload = True
+
     @classmethod
     def build(cls, settings: Settings):
         template_dirs = settings.get(['TEMPLATES', 'DIRS'])
-        loader = None  # type: jinja2.BaseLoader
-        if len(template_dirs) == 1:
-            loader = jinja2.FileSystemLoader(template_dirs[0])
+
+        package_loaders = [
+            jinja2.PrefixLoader({
+                'apistar': jinja2.PackageLoader('apistar', 'templates')
+            })
+        ]  # type: List[jinja2.BaseLoader]
+
+        if template_dirs is None:
+            filesystem_loaders = []  # type: List[jinja2.BaseLoader]
         else:
-            loader = jinja2.ChoiceLoader([
+            filesystem_loaders = [
                 jinja2.FileSystemLoader(template_dir)
                 for template_dir in template_dirs
-            ])
+            ]
 
+        loader = jinja2.ChoiceLoader(package_loaders + filesystem_loaders)
         return Templates(loader=loader)
 
 
